<?php

namespace FiloBlu\ExtEbizmartsMailChimp\Model\Api;

use Symfony\Component\Config\Definition\Exception\Exception;

class Cart extends \Ebizmarts\MailChimp\Model\Api\Cart
{

    protected $_countryFactory;

    public function __construct(
        \Ebizmarts\MailChimp\Helper\Data $helper,
        \Magento\Quote\Model\ResourceModel\Quote\CollectionFactory $quoteColletcion,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        \Magento\Customer\Model\CustomerFactory $customerFactory,
        \Ebizmarts\MailChimp\Model\Api\Product $apiProduct,
        \Ebizmarts\MailChimp\Model\Api\Customer $apiCustomer,
        \Magento\Directory\Api\CountryInformationAcquirerInterface $countryInformation,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollectionFactory,
        \Magento\Framework\Url $urlHelper,
        \Magento\Directory\Model\CountryFactory $countryFactory
    ) {
    
        $this->_countryFactory = $countryFactory;
        parent::__construct($helper, $quoteColletcion, $date, $customerFactory, $apiProduct, $apiCustomer, $countryInformation,$orderCollectionFactory,$urlHelper);
    }

    protected function _getCustomer(\Magento\Quote\Model\Quote $cart, $mailchimpStoreId, $magentoStoreId)
    {
        $api = $this->_helper->getApi($magentoStoreId);
        $customers = [];
        try {
            $customers = $api->ecommerce->customers->getByEmail($mailchimpStoreId, $cart->getCustomerEmail());
        } catch (\Mailchimp_Error $e) {
            $this->_helper->log($e->getMessage());
        }

        if (isset($customers['total_items']) && $customers['total_items'] > 0) {
            $customer = [
                'id' => $customers['customers'][0]['id']
            ];
        } else {
            if (!$cart->getCustomerId()) {
                $date = $this->_helper->getDateMicrotime();
                $customer = [
                    "id" => "GUEST-" . $date,
                    "email_address" => $cart->getCustomerEmail(),
                    "opt_in_status" => false
                ];
            } else {
                $customer = [
                    "id" => $cart->getCustomerId(),
                    "email_address" => $cart->getCustomerEmail(),
                    "opt_in_status" => $this->_apiCustomer->getOptin($magentoStoreId)
                ];
            }
        }

        $firstName = $cart->getCustomerFirstname();
        if ($firstName) {
            $customer["first_name"] = $firstName;
        }

        $lastName = $cart->getCustomerLastname();
        if ($lastName) {
            $customer["last_name"] = $lastName;
        }

        $billingAddress = $cart->getBillingAddress();
        if ($billingAddress) {
            $street = $billingAddress->getStreet();
            $address = [];
            if ($street[0]) {
                $address['address1'] = $street[0];
            }

            if (count($street) > 1) {
                $address['address1'] = $street[1];
            }

            if ($billingAddress->getCity()) {
                $address['city'] = $billingAddress->getCity();
            }

            if ($billingAddress->getRegion()) {
                $address['province'] = $billingAddress->getRegion();
            }

            if ($billingAddress->getRegionCode()) {
                $address['province_code'] = $billingAddress->getRegionCode();
            }

            if ($billingAddress->getPostcode()) {
                $address['postal_code'] = $billingAddress->getPostcode();
            }

            if ($billingAddress->getCountryId()) {
                //$country = $this->_countryInformation->getCountryInfo($billingAddress->getCountryId());
//                $countryName = $country->getFullNameLocale();
                $country = $this->_countryFactory->create()->loadByCode($billingAddress->getCountryId());
                $address['shipping_address']['country'] = $country->getName();
                $address['shipping_address']['country_code'] = $billingAddress->getCountryId();
            }

            if (count($address)) {
                $customer['address'] = $address;
            }
        }

        //company
//        if ($billingAddress->getCompany()) {
//            $customer["company"] = $billingAddress->getCompany();
//        }

        return $customer;
    }

}
