<?php

namespace FiloBlu\ExtEbizmartsMailChimp\Model\Plugin;

class Subscriber extends \Ebizmarts\MailChimp\Model\Plugin\Subscriber {

    public function __construct(
    \Ebizmarts\MailChimp\Helper\Data $helper, \Magento\Customer\Model\ResourceModel\CustomerRepository $customer, \Magento\Customer\Model\Session $customerSession, \Magento\Store\Model\StoreManagerInterface $storeManager
    ) {
        parent::__construct($helper, $customer, $customerSession, $storeManager);
    }

    public function beforeUnsubscribeCustomerById(
    $subscriber, $customerId
    ) {

        $customer = $this->_customer->getById($customerId);
        $storeId = $customer->getStoreId();

        if ($this->_helper->isMailChimpEnabled($storeId)) {
            $subscriber->loadByCustomerId($customerId);

            try {
                if ($subscriber->getMailchimpId()) {
                    $md5HashEmail = md5(strtolower($subscriber->getSubscriberEmail()));
                    $this->_api->lists->members->update($this->_helper->getDefaultList(), $md5HashEmail, null, 'unsubscribed');
//                    $subscriber->setMailchimpId('');
                }
            } catch (\Exception $e) {
                $this->_helper->log($e->getMessage());
            }
        }

        return array($customerId);
    }

    public function beforeSubscribeCustomerById(
    $subscriber, $customerId
    ) {
        $customer = $this->_customer->getById($customerId);
        $storeId = $customer->getStoreId();

        if ($this->_helper->isMailChimpEnabled($storeId)) {

            $email = $customer->getEmail();

            $subscriber->loadByEmail($email);

            $api = $this->_api;

            $status = 'subscribed';

            $mergeVars = $this->_helper->getMergeVars($subscriber, true, false);

//            if ($mergeVars) {
//                $data = array('list_id' => $this->_helper->getDefaultList(), 'email_address' => $email, 'email_type' => 'html', 'status' => $status, 'merge_fields' => $mergeVars);
//            } else {
//                $data = array('list_id' => $this->_helper->getDefaultList(), 'email_address' => $email, 'email_type' => 'html', 'status' => $status);
//            }

            try {

                $md5HashEmail = md5(strtolower($email));

                if ($subscriber->getMailchimpId()) {

                    $return = $api->lists->members->addOrUpdate($this->_helper->getDefaultList(), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);
                } else {
                    $return = $api->lists->members->addOrUpdate($this->_helper->getDefaultList(), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);
                    if (isset($return["id"])) {
                        $subscriber->setMailchimpId($return["id"]);
                    }
                }
            } catch (\Exception $e) {
                $this->_helper->log($e->getMessage());
            }
        }
//        }
        return array($customerId);
    }

    /**
     * Confirms subscriber newsletter
     *
     * @param string $code
     * @return boolean
     */
    public function beforeConfirm($subscriber, $code) {


        if ($subscriber->getCode() == $code) {

            $email = $subscriber->getEmail();
            $storeId = $this->_storeManager->getStore()->getId();

            if ($this->_helper->isMailChimpEnabled($storeId)) {

                $api = $this->_api;

                $status = 'subscribed';

                $mergeVars = $this->_helper->getMergeVars($subscriber, false, true);

//                if ($mergeVars) {
//                    $data = array('list_id' => $this->_helper->getDefaultList(), 'email_address' => $email, 'email_type' => 'html', 'status' => $status, 'merge_fields' => $mergeVars);
//                } else {
//                    $data = array('list_id' => $this->_helper->getDefaultList(), 'email_address' => $email, 'email_type' => 'html', 'status' => $status);
//                }
                try {

                    $md5HashEmail = md5(strtolower($email));

                    if ($subscriber->getMailchimpId()) {
                        $return = $api->lists->members->addOrUpdate($this->_helper->getDefaultList(), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);
                    } else {

                        $return = $api->lists->members->addOrUpdate($this->_helper->getDefaultList(), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);
                        if (isset($return["id"])) {
                            $subscriber->setMailchimpId($return["id"]);
                        }
                    }
                } catch (\Exception $e) {
                    $this->_helper->log($e->getMessage());
                }
            }
        }

        return array($code);
    }

    public function beforeSubscribe(
    $subscriber, $email
    ) {

        $storeId = $this->_storeManager->getStore()->getId();

        if ($this->_helper->isMailChimpEnabled($storeId)) {

            $isSubscribeOwnEmail = $this->_customerSession->isLoggedIn() && $this->_customerSession->getCustomerDataObject()->getEmail() == $email;

            if (!$this->_helper->isDoubleOptInEnabled($storeId) || $isSubscribeOwnEmail) {

                $api = $this->_api;

                $status = 'subscribed';

                $mergeVars = $this->_helper->getMergeVars($subscriber, false, false);

//                if ($mergeVars) {
//                    $data = array('list_id' => $this->_helper->getDefaultList(), 'email_address' => $email, 'email_type' => 'html', 'status' => $status, 'merge_fields' => $mergeVars);
//                } else {
//                    $data = array('list_id' => $this->_helper->getDefaultList(), 'email_address' => $email, 'email_type' => 'html', 'status' => $status);
//                }

                try {
                    $md5HashEmail = md5(strtolower($email));

                    if ($subscriber->getMailchimpId()) {
                        $return = $api->lists->members->addOrUpdate($this->_helper->getDefaultList(), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);
                    } else {

                        $return = $api->lists->members->addOrUpdate($this->_helper->getDefaultList(), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);

                        if (isset($return["id"])) {
                            $subscriber->setMailchimpId($return["id"]);
                        }
                    }
                } catch (\Exception $e) {
                    $this->_helper->log($e->getMessage());
                }
            }
        }
        return array($email);
    }

    public function beforeUnsubscribe(
    $subscriber
    ) {
        if ($subscriber->getMailchimpId()) {
            $md5HashEmail = md5(strtolower($subscriber->getSubscriberEmail()));
            $this->_api->lists->members->update($this->_helper->getDefaultList(), $md5HashEmail, null, 'unsubscribed');
//            $subscriber->setMailchimpId('');
        }
        return null;
    }

}
